'\"
'\" Copyright (c) 1993 The Regents of the University of California.
'\" Copyright (c) 1994-1996 Sun Microsystems, Inc.
'\"
'\" See the file "license.terms" for information on usage and redistribution
'\" of this file, and for a DISCLAIMER OF ALL WARRANTIES.
'\" 
'\" SCCS: @(#) nrematch.n 1.12 96/08/26 13:00:10
'\" 
.so man.macros
.TH nrematch n 2.0 Nre "Nre Package Commands"
.BS
'\" Note:  do not modify the .SH NAME line immediately below!
.SH NAME
nrematch \- Match a regular expression against a string
.SH SYNOPSIS
\fBpackage require nre ?2.0?\fP
.sp
\fBnrematch \fR?\fIswitches\fR? \fIexp string \fR?\fImatchVar\fR? ?\fIsubMatchVar subMatchVar ...\fR?
.BE

.SH DESCRIPTION
.PP
Determines whether the regular expression \fIexp\fR matches part or
all of \fIstring\fR and returns 1 if it does, 0 if it doesn't.
.LP
If additional arguments are specified after \fIstring\fR then they
are treated as the names of variables in which to return
information about which part(s) of \fIstring\fR matched \fIexp\fR.
\fIMatchVar\fR will be set to the range of \fIstring\fR that
matched all of \fIexp\fR.  The first \fIsubMatchVar\fR will contain
the characters in \fIstring\fR that matched the leftmost parenthesized
subexpression within \fIexp\fR, the next \fIsubMatchVar\fR will
contain the characters that matched the next parenthesized
subexpression to the right in \fIexp\fR, and so on.
.VS
Instead of using the standard regular expression package it uses
the package described in this man page.
.VE
.LP
If the initial arguments to \fBnrematch\fR start with \fB\-\fR then
they are treated as switches.  The following switches are
currently supported:
.TP 10
\fB\-nocase\fR
Causes upper-case characters in \fIstring\fR to be treated as
lower case during the matching process.
.TP 10
\fB\-indices\fR
Changes what is stored in the \fIsubMatchVar\fRs. 
Instead of storing the matching characters from \fIstring\fR,
each variable
will contain a list of two decimal strings giving the indices
in \fIstring\fR of the first and last characters in the matching
range of characters.
.TP 10
\fB\-all\fR
.VS
Instead of returning after a single match all ranges in \fIstring\fR
that match \fIexp\fR are found. Returns the number of matches found.
The \fImatchVar\fR and \fIsubMatchVars\fR are set to an empty list and as each
match is found an element is appended to the var's list.
If the \fB\-indices\fR switch is used then two elements are appended
to each list for each match found.
.VE
.TP 10
\fB\-\|\-\fR
Marks the end of switches.  The argument following this one will
be treated as \fIexp\fR even if it starts with a \fB\-\fR.
.LP
If there are more \fIsubMatchVar\fR's than parenthesized
subexpressions within \fIexp\fR, or if a particular subexpression
in \fIexp\fR doesn't match the string (e.g. because it was in a
portion of the expression that wasn't matched), then the corresponding
\fIsubMatchVar\fR will be set to ``\fB\-1 \-1\fR'' if \fB\-indices\fR
has been specified or to an empty string otherwise.

.SH "REGULAR EXPRESSIONS"
.PP
Regular expressions are implemented using Henry Spencer's package
(thanks, Henry!),
and much of the description of regular expressions below is copied verbatim
from his manual entry.
.PP
A \fIregular expression\fR is zero or more \fIbranches\fR, separated by ``|''.
It matches anything that matches one of the branches.
.PP
A \fIbranch\fR is zero or more \fIpieces\fR, concatenated.
It matches a match for the first, followed by a match for the second, etc.
.PP

A \fIpiece\fR is an \fIatom\fR possibly followed by ``*'', ``+'', ``?'',
.VS
or ``{x,y}'' which in turn might be followed by a ``?''.
.VE
A ``*'' matches a sequence of 0 or more matches of the \fIatom\fR.
A ``+'' matches a sequence of 1 or more matches of the \fIatom\fR.
A ``?'' matches a sequence of 0 or 1 matches of the \fIatom\fR.
.VS
A ``{x}'' matches a sequence of x matches of the \fIatom\fR.
A``{x,}'' matches a sequence of x or more matches of the \fIatom\fR.
A ``{x,y}'' matches a sequence of at least x and at most y matches of
the \fIatom\fR.
By default a \fIpiece\fR will match as long a sequence as
possible. However if the \fIpiece\fR constructs described above have a ``?''
after them then \fIpiece\fR will match as short a sequence as possible.
.LP
Note that the ``{x,y}'' repetition construct is only recognized if
the \fBp\fR flag is set.
.VE
.PP
An \fIatom\fR is a \fIregular expression\fR in parentheses
(matching a match for the regular expression), a \fIrange\fR (see below),
``.'' (matching any single character),
``^'' (matching the null string at the beginning of the input string),
``$'' (matching the null string at the end of the input string), 
a ``\e'' followed by a single character (matching that character
.VS
or matching something special if the \fBp\fR flag is used;
see the \fBFLAGS\fR section for details),
.VE
or a single character with no other significance (matching that character).
.PP
A \fIrange\fR is a sequence of characters enclosed in ``[]''.
It normally matches any single character from the sequence.
If the sequence begins with ``^'',
it matches any single character \fInot\fR from the rest of the sequence.
If two characters in the sequence are separated by ``\-'', this is shorthand
for the full list of ASCII characters between them
(e.g. ``[0-9]'' matches any decimal digit).
To include a literal ``]'' in the sequence, make it the first character
(following a possible ``^'').
To include a literal ``\-'', make it the first or last character.
.PP
.VS
A parentheses \fIatom\fR in which the character immediately after the ``(''
is a ``?'' is a special construct with one of the following meanings:
.LP
``(?:''\fIregexp\fR``)'' are shy groups. This groups like
``()'' but doesn't capture the text for backreferences like ``()'' does.
It matches if \fIregexp\fR matches.
.LP
``(?=''\fIregexp\fR``)'' is a non-capturing zero-width positive lookahead
assertion. It matches if \fIregexp\fR matches.
The matched text is not consumed.
.LP
``(?!''\fIregexp\fR``)'' is a non-capturing zero-width negative lookahead
assertion. It matches if \fIregexp\fR does not match.
.LP
``(?#''any text``)'' is a comment. The entire \fIatom\fR is treated as an
empty string.
.LP
``(?ipxm)'' is a used to set flags. Any combination of the flag characters
``ipxm'' are allowed. The entire \fIatom\fR is treated as an empty string.
See the \fBFLAGS\fR section for a description of each flag.
.LP
``(?|''\fIrange\fR``)'' is an alternate syntax for a character \fIrange\fR.
Its benefit is that it does not use the Tcl special characters ``[]'' to
enclose the \fIrange\fR.
.VE
.VS
.SH FLAGS
.PP
Flags can be set using a ``(?''flag-char``)'' atom. Some commands that
use regular expressions have options that set some of these same
flags. For example the -nocase option sets the i flag. The advantage
of having the flags in the regular expression itself is that they can
then be used by any command without the need to add new command
switches. It is best to set the flags at the very beginning of the
regular expression; however they apply to the entire regular expression
no matter where they appear.
.PP
The \fBi\fR flag causes case to be ignored when alphabetic characters are
compared.
.PP
The \fBm\fR flag enables multi-line mode. The ``^'' atom is changed to match
at the beginning of the string or the beginning of any line in the
string. The ``$'' atom is changed to match at the end of the string or
the end of any line in the string. The ``.'' atom is changed to match
any character except ``\en''.
.PP
The \fBx\fR flag causes white space in the regular expression to be ignored
and removed during compilation. To include literal white space as an \fIatom\fR
to be matched preceed it with a backslash ``\e''. Whitespace is only ignored
between \fIatoms, pieces, branches, and regular expressions\fR.
It is not ignored in \fIranges\fR or in any other complex \fIatom\fR.
The white space includes comments where a comment starts with a ``#''
and continues to the end of the line.
.PP
The \fBp\fR flag enables extra escape sequences and constructs to be
recognized. See the \fBBACKWARDS COMPATIBILITY\fR section for why these
constructs are not enabled by default. The following are enabled:
.VE
.IP \fB\ew\fR
.VS
Match a "word" character (alphanumeric plus "_").
The alphanumeric characters are determined using isalnum(3)
which can vary depending on the locale.
.VE
.IP \fB\eW\fR
.VS
Match a non-word character.
.VE
.IP \fB\es\fR
.VS
Match a whitespace character.
The whitespace characters are determined using isspace(3)
which can vary depending on the locale.
.VE
.IP \fB\eS\fR
.VS
Match a non-whitespace character.
.VE
.IP \fB\ed\fR
.VS
Match a digit character.
The digit characters are determined using isdigit(3)
which can vary depending on the locale.
.VE
.IP \fB\eD\fR
.VS
Match a non-digit character.
.VE
.IP \fB\eb\fR
.VS
Zero-width assertion matches a word boundary.
Current character matches \fB\ew\fR and previous character matches \fB\eW\fR or
current character matches \fB\eW\fR and previous character matches \fB\ew\fR.
The position before the first character in the string and after the
last character match \fB\eW\fR.
.VE
.IP \fB\eB\fR
.VS
Zero-width assertion matches a non-word boundary.
Current character matches \fB\ew\fR and previous character matches \fB\ew\fR or
current character matches \fB\eW\fR and previous character matches \fB\eW\fR.
The position before the first character in the string and after the
last character match \fB\eW\fR.
.VE
.IP \fB\e<\fR 
.VS
Zero-width assertion matches start of word.
Current character matches \fB\ew\fR and previous character matches \fB\eW\fR.
The position before the first character in the string and after the
last character match \fB\eW\fR.
.VE
.IP \fB\e>\fR 
.VS
Zero-width assertion matches end of word.
Current character matches \fB\eW\fR and previous character matches \fB\ew\fR.
The position before the first character in the string and after the
last character match \fB\eW\fR.
.VE
.IP \fB\eA\fR
.VS
Zero-width assertion matches only at beginning of string even
if \fBm\fR flag.
.VE
.IP \fB\eZ\fR
.VS
Zero-width assertion matches only at end of string even if \fBm\fR flag.
.VE
.IP \fB\eG\fR
.VS
Zero-width assertion matches only where previous \fB\-all\fR match left off.
.VE
.IP \fB\eQ\fR
.VS
Quote mode. All characters following are treated as literal text 
until a \fB\eE\fR or the end of the regular expression.
.VE
.IP \fB\eE\fR
.VS
End quote mode.
.VE
.IP \fB\e\fR\fInum\fR
.VS
Backreference to the \fInum\fR'th captured substring. The value of \fInum\fR
must not be greater than the number of captured substrings to the left
of the backreference.  The text from the backreference is inserted
into the regular expression and is always treated as literal text.
.VE
.IP \fB\e\fR\fImeta\fR
.VS
If the ``\e'' is followed by a regular expression meta character then
the meta character is treated as literal text. The meta chararacters
are: ``\fB\e*+?()|[]{}^$\fR''. If ``\e'' is followed by anything else
the regexp compiler will raise an error.
.VE
.IP \fB{x,y}\fR
.VS
This \fIpiece\fR construct is a repetition operator and is described above
in the \fIpiece\fR paragraph.
.VE
.SH "CHOOSING AMONG ALTERNATIVE MATCHES"
.PP
In general there may be more than one way to match a regular expression
to an input string.  For example, consider the command
.CS
\fBnrematch  (a*)b*  aabaaabb  x  y\fR
.CE
Considering only the rules given so far, \fBx\fR and \fBy\fR could
end up with the values \fBaabb\fR and \fBaa\fR, \fBaaab\fR and \fBaaa\fR,
\fBab\fR and \fBa\fR, or any of several other combinations.
To resolve this potential ambiguity \fBnrematch\fR chooses among
alternatives using the following rules apply in decreasing
order of priority:
.IP [1]
If a regular expression could match two different parts of an input string
then it will match the one that begins earliest.
.IP [2]
If a regular expression contains \fB|\fR operators then the leftmost
matching sub-expression is chosen.
.IP [3]
In \fB*\fR, \fB+\fR, \fB?\fR, and
.VS
\fB{x,y}\fR
.VE
constructs,
longer matches are chosen in preference to shorter ones.
These operators are often called \fIgreedy\fR because they
match the longest possible string that allows the entire regular
expression to match.
.VS
In \fB*?\fR, \fB+?\fR, \fB??\fR, and \fB{x,y}?\fR constructs,
shorter matches are chosen in preference to longer ones.
These operators are often called \fIlazy\fR because they match
the shortest possible string that allows the entire regular expression
to match.
.VE
.IP [4]
In sequences of expression components the components are considered
from left to right.
.LP
In the example from above, \fB(a*)b*\fR matches \fBaab\fR:  the \fB(a*)\fR
portion of the pattern is matched first and it consumes the leading
\fBaa\fR; then the \fBb*\fR portion of the pattern consumes the
next \fBb\fR.  Or, consider the following example:
.CS
\fBnrematch  (ab|a)(b*)c  abc  x  y  z\fR
.CE
After this command \fBx\fR will be \fBabc\fR, \fBy\fR will be
\fBab\fR, and \fBz\fR will be an empty string.
Rule 4 specifies that \fB(ab|a)\fR gets first shot at the input
string and Rule 2 specifies that the \fBab\fR sub-expression
is checked before the \fBa\fR sub-expression.
Thus the \fBb\fR has already been claimed before the \fB(b*)\fR
component is checked and \fB(b*)\fR must match an empty string.
.VS
.SH LIMITS
.PP
The maximum number of capturing subexpressions ``()'' in a single
regular expression is 255. This limit does not apply to the
non-capturing ``(?:)''.
.PP
A compiled regular expression is limited in size to 32678 bytes. If
during compilation it is discovered that the regular expression
requires more memory then the operation will fail with the error:
``regexp too big''.
.PP
The counts in the repetition construct ``{x,y}'' must be greater than
or equal to zero and less than or equal to 255.
.PP
The maximum number of unique \fIranges\fR in a regular expression is 64.
.VE
.VS
.SH "BACKWARDS COMPATIBILITY"
.PP
Regular expressions from previous releases of Tcl should behave
exactly the same. The following new constructs:
.LP
     (?...),   *?,   +?,   and   ??
.LP
will cause compilation errors in older regular expressions so they are
always recognized in new regular expressions.
.PP
All the other new constructs would have meant something else in older
regular expressions. So they always have the old meaning unless you
turn on one of the new flags.  For example you need to start a regular
expression with \fB(?p)\fR if you want to use the new ``\e'' sequences or
the ``{x,y}'' repetition construct.
.VE
.VS
.SH "PERFORMANCE INFORMATION"
The first time a regular expressions is used it is compiled into
a Tcl object. The next time that object needs to be used as a
regular expression the compilation step will not be needed if the
object still exists and is still a regular expression.
So if the regular expression is a constant string:
.CS
nrematch {abc|def|zeq} $str
.CE
then the first time the above command is executed the string
constant object is converted to a regular expression object
and will remain so giving a performance boost.
.PP
However if the regular expression string is not constant:
.CS
nrematch "$W1|$W2|$W3" $str
.CE
then the string object will need to be recreated each time the
above command executes.
.PP
If instead you stored the regular expression string into a variable
then the regular expression object would remain and not need to
be recreated each time:
.CS
set re "$W1|$W2|$W3"
proc foo {} {
    global re
    nrematch $re $str
}
.CE
If it is a complex regular expression used in more than one place
this can be a win in both time and space.
.PP
It is best to use \fB(?i)\fR instead of \fB\-nocase\fR if you can
because then the text of the regular expression object describes
its state. 
.PP
If you do not need the \fImatchVar\fR or a \fIsubMatchVar\fR then you
can set that argument to an empty string ``{}''.
This tells \fBnrematch\fR to not bother setting a variable to that
particular captured subexpression.
.VE
.VS
.SH "BINARY CLEAN"
The new regular expression compiler and matcher are binary clean.
This means that it is ok for the regular expression and the string
being matched to contain binary data including null bytes.
.VE
.VS
.SH EXAMPLES
To match a number if not followed by a period:
.CS
\fBnrematch {[0-9]+(?![.])} $str\fR
.CE
.PP
To match a number if followed by something other than a period:
.CS
\fBnrematch {[0-9]+(?=[^.])} $str\fR
.CE
.PP
To match an item that contains only letters, but not all uppercase:
.CS
\fBnrematch {^(?![A-Z]*$)[a-zA-Z]*$} $str\fR
.CE
.PP
To see if a string contains both 'this' and 'that':
.CS
\fBnrematch {^(?=.*?this)(?=.*?that)} $str\fR
.CE

.VE
.SH KEYWORDS
match, nre, regular expression, string
